#!/usr/bin/env python
""" This is an example of a script that creates and applies a custom made
GGRAPH template (*.sgt) based on the content of the active dataset.
The template is saved to the "c:\temp\" folder.

Usage:
You can either
  1)run this file from a python IDE or
  2)enclose it within a BEGIN PROGRAM - END PROGRAM block and run it from SPSS's
    syntax editor.

Author: Raynald Levesque raynald@spsstools.net
Date: April 30, 2007
"""

import spss
xDrive = spss.PyInvokeSpss.IsXDriven() # xDrive=1 means spo not available

# Create a data file for illustration purposes.
# When running SPSS from a python IDE, a new instance of SPSS is used each time
# a python script is run. This implies that the seed of the Pseudo random number
# generator is always the same. Using SET SEED=RANDOM avoids this problem.

cmd = r"""
SET SEED = RANDOM.
INPUT PROGRAM.
 COMPUTE mean=RND(UNIFORM(1000)).
 LEAVE mean.
 LOOP #cnt=1 TO 1000.
  COMPUTE draw=RV.NORMAL(mean,50).
  END CASE.
 END LOOP.
END FILE.
END INPUT PROGRAM.
EXECUTE. """
spss.Submit(cmd)

oms = '* viewer exists, no need to export to jpg format.'
if xDrive:  # use OMS to export GGRAPH as a jpg (viewer is not available)
    oms = r"""
    OMS SELECT CHARTS
      /IF COMMAND=['GGraph']
      /DESTINATION  FORMAT =HTML
       IMAGES=YES
       IMAGEFORMAT=JPG
       IMAGEROOT='histogram'
       OUTFILE='C:/temp/histogram.html'
       VIEWER = NO.
    """

# get mean of distribution (first variable of first case)
dataCursor=spss.Cursor()
mean=dataCursor.fetchone()[0]
dataCursor.close()
print "mean=",mean

# Define temporary template with reference line at the mean
# To obtain the initial template, the vertical reference line was added manually
# then the template was saved. This template was loaded using Notepad and pasted
# below (adding a few carriage returns to make it easier to work with)
# Note that the Distribution's mean value is inserted within the template

template = r"""<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<template SPSS-Version="2.2" date="2007-04-30" description=""
selectPath="900 " xmlns="http://xml.spss.com/spss/visualization"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xsi:schemaLocation="http://xml.spss.com/spss/visualization
http://xml.spss.com/spss/visualization/vizml-template-3.0.xsd">
<addReferenceLine styleOnly="false" x="%(mean)u" xcategorical="false"/>
</template>  """ % locals()

# Of course, an alternative method would be to read a base sgt from disk, replace the
# required section and save the result to "temp.sgt"

#Write .sgt file to disk
f = open(r'C:/temp/temp.sgt','w')
try:
    f.write(template)
finally:
    f.close()

# Display histogram of sample with reference line at Distribution's mean
cmd = r"""
%(oms)s
* Chart Builder.
GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=draw MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE TEMPLATE=["C:/temp/temp.sgt"].
BEGIN GPL
 SOURCE: s=userSource(id("graphdataset"))
 DATA: draw=col(source(s), name("draw"))
 GUIDE: axis(dim(1), label("draw"))
 GUIDE: axis(dim(2), label("Frequency"))
 GUIDE: text.title(label("Sample is based on a N(%(mean)u,50) "))
 ELEMENT: interval(position(summary.count(bin.rect(draw))), shape.interior(shape.square))
END GPL.
OMSEND.  """ % locals()

spss.Submit(cmd)
print '*** end of script'
